source("modules/Anomalies/tmp/CST_Anomaly.R")

# Compute the hcst, obs and fcst anomalies with or without cross-validation
# and return them, along with the hcst and obs climatologies.

compute_anomalies <- function(recipe, data) {

  if (recipe$Analysis$Workflow$Anomalies$compute) {
    if (recipe$Analysis$Workflow$Anomalies$cross_validation) {
      cross <- TRUE
      cross_msg <- "with"
    } else {
      cross <- FALSE
      cross_msg <- "without"
    }
    original_dims <- dim(data$hcst$data)
    anom <- CST_Anomaly(data$hcst, data$obs,
			cross = cross,
			memb = TRUE,
			memb_dim = 'ensemble',
			dim_anom = 'syear',
			dat_dim = c('dat', 'ensemble'),
			ftime_dim = 'time')

    anom$exp$data <- Reorder(anom$exp$data, names(original_dims))
    anom$obs$data <- Reorder(anom$obs$data, names(original_dims))

    hcst_fullvalue <- data$hcst
    obs_fullvalue <- data$obs
    
    ## Maybe just use s2dv::Clim() here?
    clim <- s2dv::Clim(data$hcst$data, data$obs$data,
        	       time_dim = "syear",
        	       dat_dim = c("dat", "ensemble"),
        	       memb = TRUE,
        	       memb_dim = "ensemble",
        	       ftime_dim = "time")
    clim_hcst <- InsertDim(clim$clim_exp, posdim = 1, lendim = 1,
                           name = "syear")
    clim_hcst <- Reorder(clim_hcst, names(original_dims))
    # clim_obs$data <- InsertDim(clim$clim_obs,
    #     		       posdim = 1, lendim = 1, name = "syear")
    # clim_obs$data <- Reorder(clim_obs$data, names(original_dims))

    data$hcst <- anom$exp
    data$obs <- anom$obs
    remove(anom, clim)

    ## TODO: Compute forecast anomaly field
    if (!is.null(data$fcst)) {
      warn(recipe$Run$logger,
	   "fcst anomalies are a work in progress...")
      mean_clim_hcst <- MeanDims(clim_hcst, dims = "ensemble", drop = T)
      dims <- dim(mean_clim_hcst)
      mean_clim_hcst <- rep(mean_clim_hcst, dim(data$fcst$data)[['ensemble']])
      dim(mean_clim_hcst) <- c(dims,
			       ensemble = dim(data$fcst$data)[['ensemble']])
      data$fcst$data <- data$fcst$data - mean_clim_hcst
      info(recipe$Run$logger,
	   "The anomalies have been computed, ", cross_msg, " cross-validation.
	   The original full fields are returned in $hcst_full.val and 
	   $obs_full.val.")
    }

    info(recipe$Run$logger, "##### ANOMALIES COMPUTED SUCCESSFULLY #####")

  } else {
    warn(recipe$Run$logger, "The Anomalies module has been called, but 
	 recipe parameter Analysis:Variables:anomaly is set to FALSE.
	 The full fields will be returned.")
    hcst_fullvalue <- NULL
    obs_fullvalue <- NULL
    info(recipe$Run$logger, "##### ANOMALIES NOT COMPUTED #####")
  }

  return(list(hcst = data$hcst, obs = data$obs, fcst = data$fcst,
	      hcst_full.val = hcst_fullvalue, obs_full.val = obs_fullvalue))

}






